import sys
from PyQt5.QtCore import  QObject, QRunnable, QThreadPool, pyqtSignal


def async_execute(task):
    QThreadPool.globalInstance().start(task)


class TaskSignal(QObject):
    callback = pyqtSignal(object, bool)

    def __init__(self, callback, parent=None):
        super(TaskSignal, self).__init__(parent)
        self.callback.connect(callback)


class Task(QRunnable):

    def __init__(self, method, args, callback):
        super(Task, self).__init__()

        self.method = method
        self.args = args
        self.canceled = False
        self.signal = None
        self.callback = callback
        self.setAutoDelete(True)

    def cancel(self):
        self.canceled = True

    def signal_success(self, result):
        self.signal.callback.emit(result, True)

    def signal_failure(self, result):
        self.signal.callback.emit(result, False)

    def signal_canceled(self):
        self.signal.callback.emit("Task canceled.", False)

    def run(self):
        self.signal = TaskSignal(self.callback)
        if self.canceled:
            self.signal_canceled()
            return
        try:
            result = self.method(*self.args)
            if self.canceled:
                self.signal_canceled()
                return
            self.signal_success(result)
        except:
            (etype, value, traceback) = sys.exc_info()
            if self.canceled:
                self.signal_canceled()
                return
            self.signal_failure(str(value))
        finally:
            self.cleanup()

    def cleanup(self):
        if self.signal is not None:
            self.signal.deleteLater()



import sys
import traceback
from PyQt5 import QtCore
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QApplication, QStyleFactory, QMessageBox
from bagit import BagError
from bdbag import get_typed_exception
from bdbag_gui.ui import main_window as mw
from bdbag_gui import resources


def excepthook(etype, value, tb):
    sys.stderr.write(get_typed_exception(value))
    if isinstance(value, BagError) or isinstance(value, RuntimeError):
        return
    traceback.print_tb(tb)
    msg = QMessageBox()
    msg.setText(str(value))
    msg.setStandardButtons(QMessageBox.Close)
    msg.setWindowTitle("Unhandled Exception: %s" % etype.__name__)
    msg.setIcon(QMessageBox.Critical)
    msg.setDetailedText('\n'.join(traceback.format_exception(etype, value, tb)))
    msg.exec_()


def main():
    sys.excepthook = excepthook
    QApplication.setDesktopSettingsAware(False)
    QApplication.setStyle(QStyleFactory.create("Fusion"))
    app = QApplication(sys.argv)
    app.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps)
    app.setWindowIcon(QIcon(":/images/bag.png"))
    mainWindow = mw.MainWindow()
    mainWindow.show()
    ret = app.exec()
    return ret


if __name__ == '__main__':
    sys.exit(main())



from PyQt5 import QtCore
from PyQt5.QtCore import pyqtSignal

from bdbag import bdbag_api as bdb
from bdbag_gui.impl.async_task import Task, async_execute


class BagTask(QtCore.QObject):
    status_update_signal = pyqtSignal(str, bool)
    progress_update_signal = pyqtSignal(int, int)

    def __init__(self, parent=None):
        super(BagTask, self).__init__(parent)
        self.task = None

    def start(self):
        async_execute(self.task)

    def cancel(self):
        self.task.cancel()

    def terminate(self):
        self.task.terminate()

    def set_status(self, status, success):
        self.status_update_signal.emit(status, success)

    def result_callback(self, result, success):
        self.set_status(result, success)

    def progress_callback(self, current, maximum):
        if self.task.canceled:
            return False

        self.progress_update_signal.emit(current, maximum)
        return True


class BagCreateOrUpdateTask(BagTask):

    def __init__(self, parent=None):
        super(BagCreateOrUpdateTask, self).__init__(parent)
        self.update = False

    def result_callback(self, result, success):
        status = "Bag %s completed successfully." % ("update" if self.update else "creation") if success else \
            "Bag %s error: %s" % ("update" if self.update else "creation", result)
        self.set_status(status, success)

    def createOrUpdate(self, bag_path, update, config_file):
        self.update = update
        self.task = Task(bdb.make_bag,
                         [bag_path, ['md5', 'sha256'], update, True, False, None, None, None, config_file],
                         self.result_callback)
        self.start()


class BagRevertTask(BagTask):

    def __init__(self, parent=None):
        super(BagRevertTask, self).__init__(parent)

    def result_callback(self, result, success):
        status = "Bag reverted successfully." if success else "Bag reversion failed: %s" % result
        self.set_status(status, success)

    def revert(self, bag_path):
        self.task = Task(bdb.revert_bag,
                         [bag_path],
                         self.result_callback)
        self.start()


class BagValidateTask(BagTask):

    def __init__(self, parent=None):
        super(BagValidateTask, self).__init__(parent)

    def result_callback(self, result, success):
        status = "Bag validation complete." if success else "Bag validation error: %s" % result
        self.set_status(status, success)

    def validate(self, bag_path, fast, config_file):
        self.task = Task(bdb.validate_bag,
                         [bag_path, fast, self.progress_callback, config_file],
                         self.result_callback)
        self.start()


class BagFetchTask(BagTask):

    def __init__(self, parent=None):
        super(BagFetchTask, self).__init__(parent)

    def result_callback(self, result, success):
        status = ("Bag fetch complete: All file references resolved successfully." if result else
                  "Bag fetch incomplete: Some file references were not resolved.") if success else \
            "Bag fetch error: %s" % result
        self.set_status(status, success)

    def fetch(self, bag_path, fetch_all, keychain_file, config_file):
        self.task = Task(bdb.resolve_fetch,
                         [bag_path, fetch_all, self.progress_callback, keychain_file, config_file],
                         self.result_callback)
        self.start()


class BagArchiveTask(BagTask):

    def __init__(self, parent=None):
        super(BagArchiveTask, self).__init__(parent)

    def result_callback(self, result, success):
        status = "Bag archive complete." if success else "Bag archive error: %s" % result
        self.set_status(status, success)

    def archive(self, bag_path, archiver):
        self.task = Task(bdb.archive_bag,
                         [bag_path, archiver],
                         self.result_callback)
        self.start()


class BagExtractTask(BagTask):

    def __init__(self, parent=None):
        super(BagExtractTask, self).__init__(parent)

    def result_callback(self, result, success):
        status = "File extraction complete." if success else "File extraction error: %s" % result
        self.set_status(status, success)

    def extract(self, bag_path, output_path=None):
        self.task = Task(bdb.extract_bag,
                         [bag_path, output_path],
                         self.result_callback)
        self.start()


class BagMaterializeTask(BagTask):

    def __init__(self, parent=None):
        super(BagMaterializeTask, self).__init__(parent)

    def result_callback(self, result, success):
        status = "Bag materialization complete." if success else "Bag materialization error: %s" % result
        self.set_status(status, success)

    def materialize(self, bag_path, output_path=None):
        self.task = Task(bdb.materialize,
                         [bag_path, output_path, self.progress_callback, self.progress_callback],
                         self.result_callback)
        self.start()



import os
import json
import logging
import platform

from PyQt5.Qt import PYQT_VERSION_STR
from PyQt5.QtCore import Qt, QMetaObject, QModelIndex, QThreadPool, QTimer, QMutex, pyqtSlot
from PyQt5.QtWidgets import QApplication, QMainWindow, QWidget, QAction, QMenu, QMenuBar, QMessageBox, QStyle, \
    QProgressBar, QToolBar, QStatusBar, QVBoxLayout, QTreeView, QFileSystemModel, qApp
from PyQt5.QtGui import QIcon
from bdbag import VERSION as BDBAG_VERSION, BAGIT_VERSION, BAGIT_PROFILE_VERSION, bdbag_api as bdb
from bdbag_gui import resources, VERSION
from bdbag_gui.ui import log_widget, options_window
from bdbag_gui.ui.options_window import DEFAULT_OPTIONS, DEFAULT_OPTIONS_FILE
from bdbag_gui.impl import async_task, bag_tasks


# noinspection PyBroadException,PyArgumentList
class MainWindow(QMainWindow):

    def __init__(self):
        super(MainWindow, self).__init__()
        self.currentTask = None
        self.currentTaskMutex = QMutex()
        self.options = DEFAULT_OPTIONS
        self.ui = MainWindowUI()
        self.ui.setup_ui(self)
        self.ui.logTextBrowser.widget.log_update_signal.connect(self.updateLog)
        self.ui.logTextBrowser.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s'))
        logging.getLogger().addHandler(self.ui.logTextBrowser)
        logging.getLogger().setLevel(logging.INFO)

        self.fileSystemModel = QFileSystemModel()
        self.fileSystemModel.setReadOnly(False)
        self.ui.treeView.setModel(self.fileSystemModel)
        self.ui.treeView.setAnimated(True)
        self.ui.treeView.setAcceptDrops(True)
        self.ui.treeView.setSortingEnabled(True)
        self.ui.treeView.sortByColumn(0, Qt.AscendingOrder)
        self.ui.treeView.setColumnWidth(0, 300)
        self.ui.treeView.setRootIndex(self.fileSystemModel.setRootPath(self.fileSystemModel.myComputer()))

        self.loadOptions()
        self.enableControls(True)
        self.ui.actionCreateOrUpdate.setEnabled(False)

    def loadOptions(self, options_file=DEFAULT_OPTIONS_FILE):
        if not os.path.isfile(options_file):
            logging.debug("Creating default options file: %s" % options_file)
            self.saveOptions(options_file)

        if os.path.isfile(options_file):
            logging.debug("Loading options file from: %s" % options_file)
            with open(options_file) as of:
                options = of.read()
        else:
            options = json.dumps(DEFAULT_OPTIONS)
            logger.warning("Unable to read options file: [%s]. Using internal defaults." % config_file)

        self.options = json.loads(options)

    def saveOptions(self, options_file=DEFAULT_OPTIONS_FILE):
        logging.debug("Writing options file: %s" % options_file)
        try:
            options_path = os.path.dirname(options_file)
            if not os.path.isdir(options_path):
                try:
                    os.makedirs(options_path, mode=0o750)
                except OSError as error:
                    if error.errno != errno.EEXIST:
                        raise
            with open(options_file, 'w') as of:
                of.write(json.dumps(self.options, indent=4, sort_keys=True))
        except Exception as e:
            logger.warning("Unable to write options file: [%s]. Error: %s" % (options_file, e))

    def checkIfBag(self, silent=False):
        current_path = self.getCurrentPath()
        if not current_path:
            return False
        current_type = self.fileSystemModel.type(self.ui.treeView.currentIndex())
        if current_type == "Drive":
            is_bag = False
        else:
            if os.path.isdir(current_path):
                QApplication.setOverrideCursor(Qt.WaitCursor)
                is_bag = bdb.is_bag(current_path)
                QApplication.restoreOverrideCursor()
                if not silent:
                    self.updateStatus("The directory [%s] is%s a bag." % (current_path, "" if is_bag else " NOT"), True)
            else:
                is_bag = False

        return is_bag

    def checkIfArchive(self, silent=False):
        is_file_archive = False
        current_path = self.getCurrentPath()
        if current_path and os.path.isfile(current_path):
            # simple test based on extension only
            ext = os.path.splitext(current_path)[1]
            if ext in (".zip", ".tar", ".tgz", ".gz", ".bz2"):
                is_file_archive = True

        if is_file_archive and not silent:
            self.updateStatus("The file [%s] is a supported archive format." % current_path, True)

        return is_file_archive

    def setCurrentTask(self, task, can_cancel=True):
        if not self.currentTaskMutex.tryLock(10):
            return False
        self.currentTask = task
        self.bagTaskTriggered(can_cancel)
        return True

    def clearCurrentTask(self):
        if not self.currentTask:
            return
        self.currentTask = None
        self.currentTaskMutex.unlock()

    def getCurrentPath(self):
        return os.path.normpath(os.path.abspath(self.fileSystemModel.filePath(self.ui.treeView.currentIndex())))

    def disableControls(self, can_cancel=True):
        self.ui.actionCancel.setEnabled(can_cancel)
        self.ui.treeView.setEnabled(False)
        self.ui.actionCreateOrUpdate.setEnabled(False)
        self.ui.actionRevert.setEnabled(False)
        self.ui.actionMaterialize.setEnabled(False)
        self.ui.actionFetchMissing.setEnabled(False)
        self.ui.actionFetchAll.setEnabled(False)
        self.ui.actionValidateFast.setEnabled(False)
        self.ui.actionValidateFull.setEnabled(False)
        self.ui.actionArchive.setEnabled(False)
        self.ui.actionDelete.setEnabled(False)
        self.ui.actionOptions.setEnabled(False)

    def enableControls(self, silent=False):
        is_bag = self.checkIfBag(silent)
        is_file_archive = self.checkIfArchive(silent)
        current_path = self.getCurrentPath()
        current_type = self.fileSystemModel.type(self.ui.treeView.currentIndex())
        self.ui.treeView.setEnabled(True)
        self.ui.actionOptions.setEnabled(True)
        self.ui.actionCancel.setEnabled(False)
        self.ui.actionDelete.setEnabled(False if (not current_type or "Drive" == current_type) else True)
        self.ui.toggleCreateOrUpdate(self, is_bag)
        self.ui.actionCreateOrUpdate.setEnabled(
            (os.path.isdir(current_path) and "Drive" != current_type) if current_path else False)
        self.ui.actionRevert.setEnabled(is_bag)
        self.ui.actionMaterialize.setEnabled(is_bag or is_file_archive)
        self.ui.actionFetchMissing.setEnabled(is_bag)
        self.ui.actionFetchAll.setEnabled(is_bag)
        self.ui.actionValidateFast.setEnabled(is_bag)
        self.ui.actionValidateFull.setEnabled(is_bag)
        self.ui.toggleArchiveOrExtract(self, is_bag, is_file_archive)

    def selectionChanged(self):
        self.ui.statusBar.clearMessage()
        self.ui.progressBar.reset()
        self.ui.logTextBrowser.widget.clear()
        self.enableControls()

    def closeEvent(self, event):
        self.cancelTasks()
        event.accept()

    def cancelTasks(self):
        if not self.currentTask:
            return

        self.disableControls()
        self.currentTask.cancel()
        self.statusBar().showMessage("Waiting for background tasks to terminate...")

        while True:
            qApp.processEvents()
            if not self.currentTask and QThreadPool.globalInstance().waitForDone(10):
                break

        self.statusBar().showMessage("All background tasks terminated successfully.")

    @pyqtSlot()
    def bagTaskTriggered(self, can_cancel=True):
        self.ui.progressBar.reset()
        self.ui.progressBar.setTextVisible(can_cancel)
        self.ui.logTextBrowser.widget.clear()
        self.disableControls(can_cancel)

    @pyqtSlot(str)
    def updateStatus(self, status, success=True):
        if success:
            logging.info(status)
        else:
            logging.error(status)
        self.statusBar().showMessage(status)

    @pyqtSlot(str, bool)
    def updateUI(self, status, success=True):
        self.updateStatus(status, success)
        self.clearCurrentTask()
        self.enableControls(True)

    @pyqtSlot(str)
    def updateLog(self, text):
        self.ui.logTextBrowser.widget.appendPlainText(text)

    @pyqtSlot(int, int)
    def updateProgress(self, current, maximum):
        self.ui.progressBar.setRange(1, maximum)
        self.ui.progressBar.setValue(current)
        self.ui.progressBar.repaint()

    @pyqtSlot(bool)
    def on_actionCreateOrUpdate_triggered(self):
        current_path = self.getCurrentPath()
        if not current_path:
            return

        if os.path.dirname(current_path) == current_path:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Critical)
            msg.setWindowTitle("Operation not allowed")
            msg.setText("Bagging of root filesystems prohibited.")
            msg.setInformativeText(
                "The current selection is a filesystem root. It is not possible to create a bag of a filesystem root.")
            msg.exec_()
            return

        if not self.setCurrentTask(bag_tasks.BagCreateOrUpdateTask()):
            return
        self.currentTask.status_update_signal.connect(self.updateUI)
        self.currentTask.createOrUpdate(current_path, self.checkIfBag(), self.options.get("bag_config_file_path"))

    @pyqtSlot(bool)
    def on_actionRevert_triggered(self):
        current_path = self.getCurrentPath()
        if not current_path:
            return

        msg = QMessageBox()
        msg.setIcon(QMessageBox.Warning)
        msg.setWindowTitle("Confirm Action")
        msg.setText("Are you sure you want to revert this bag directory?")
        msg.setInformativeText("Reverting a bag directory will cause all manifests to be deleted including fetch.txt, "
                               "if present.\n\nIf a bag contains remote file references, these files will no longer be "
                               "resolvable.\n\nIt is recommended that you either resolve any remote files prior to "
                               "reverting such a bag, or make a backup copy of it first.")
        msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
        ret = msg.exec_()
        if ret == QMessageBox.Cancel:
            return

        if not self.setCurrentTask(bag_tasks.BagRevertTask()):
            return
        self.currentTask.status_update_signal.connect(self.updateUI)
        self.currentTask.revert(current_path)

    @pyqtSlot(bool)
    def on_actionMaterialize_triggered(self):
        current_path = self.getCurrentPath()
        if not current_path:
            return

        if not self.setCurrentTask(bag_tasks.BagMaterializeTask()):
            return
        self.currentTask.status_update_signal.connect(self.updateUI)
        self.currentTask.progress_update_signal.connect(self.updateProgress)
        self.currentTask.materialize(current_path, self.options.get("archive_extract_dir"))
        self.updateStatus("Materialize initiated for bag: [%s] -- Please wait..." % current_path)

    @pyqtSlot(bool)
    def on_actionArchive_triggered(self):
        current_path = self.getCurrentPath()
        if not current_path:
            return
        is_bag = self.checkIfBag()
        is_file_archive = self.checkIfArchive()
        if is_file_archive:
            if not self.setCurrentTask(bag_tasks.BagExtractTask(), can_cancel=False):
                return
            self.currentTask.status_update_signal.connect(self.updateUI)
            self.currentTask.extract(current_path, self.options.get("archive_extract_dir"))
            self.updateStatus("Extracting file: [%s] -- Please wait..." % current_path)
        elif is_bag:
            archive_format = self.options.get("archive_format", "zip")
            if not self.setCurrentTask(bag_tasks.BagArchiveTask()):
                return
            self.currentTask.status_update_signal.connect(self.updateUI)
            self.currentTask.archive(current_path, archive_format)
            self.updateStatus("Archive (%s) initiated for bag: [%s] -- Please wait..." %
                              (archive_format.upper(), current_path))

    @pyqtSlot(bool)
    def on_actionValidateFast_triggered(self):
        current_path = self.getCurrentPath()
        if not current_path:
            return
        if not self.setCurrentTask(bag_tasks.BagValidateTask()):
            return
        self.currentTask.status_update_signal.connect(self.updateUI)
        self.currentTask.validate(current_path, True, self.options.get("bag_config_file_path"))

    @pyqtSlot(bool)
    def on_actionValidateFull_triggered(self):
        current_path = self.getCurrentPath()
        if not current_path:
            return
        if not self.setCurrentTask(bag_tasks.BagValidateTask()):
            return
        self.currentTask.status_update_signal.connect(self.updateUI)
        self.currentTask.progress_update_signal.connect(self.updateProgress)
        self.currentTask.validate(current_path, False, self.options.get("bag_config_file_path"))
        self.updateStatus("Full validation initiated for bag: [%s] -- Please wait..." % current_path)

    @pyqtSlot(bool)
    def on_actionFetchAll_triggered(self):
        current_path = self.getCurrentPath()
        if not current_path:
            return
        if not self.setCurrentTask(bag_tasks.BagFetchTask()):
            return
        self.currentTask.status_update_signal.connect(self.updateUI)
        self.currentTask.progress_update_signal.connect(self.updateProgress)
        self.currentTask.fetch(current_path,
                               True,
                               self.options.get("bag_keychain_file_path"),
                               self.options.get("bag_config_file_path"))
        self.updateStatus("Fetch all initiated for bag: [%s] -- Please wait..." % current_path)

    @pyqtSlot(bool)
    def on_actionFetchMissing_triggered(self):
        current_path = self.getCurrentPath()
        if not current_path:
            return
        if not self.setCurrentTask(bag_tasks.BagFetchTask()):
            return
        self.currentTask.status_update_signal.connect(self.updateUI)
        self.currentTask.progress_update_signal.connect(self.updateProgress)
        self.currentTask.fetch(current_path,
                               False,
                               self.options.get("bag_keychain_file_path"),
                               self.options.get("bag_config_file_path"))
        self.updateStatus("Fetch missing initiated for bag: [%s] -- Please wait..." % current_path)

    @pyqtSlot(QModelIndex)
    def on_treeView_clicked(self, index):
        self.selectionChanged()

    @pyqtSlot(bool)
    def on_actionOptions_triggered(self):
        options_window.OptionsDialog.getOptions(self)

    @pyqtSlot(bool)
    def on_actionCancel_triggered(self):
        self.cancelTasks()
        self.ui.progressBar.reset()
        self.enableControls(True)

    @pyqtSlot()
    def on_actionDelete_triggered(self):
        is_dir = self.fileSystemModel.isDir(self.ui.treeView.currentIndex())
        obj = "Directory" if is_dir else "File"
        current_path = self.getCurrentPath()
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Warning)
        msg.setWindowTitle("%s Deletion Warning" % obj)
        msg.setText("The %s \"%s\" will be deleted." % (obj.lower(), current_path))
        msg.setInformativeText("You are about to DELETE a %s. "
                               "This operation is permanent and CANNOT BE UNDONE.\n\nAre you sure?" % obj.lower())
        msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Abort)
        ret = msg.exec_()
        if ret == QMessageBox.Ok:
            result = self.fileSystemModel.remove(self.ui.treeView.currentIndex())
            self.updateStatus("%s: [%s]" % (("Successfully deleted" if result else "Failed to delete"),
                                            current_path), result)
            qApp.processEvents()
            QTimer.singleShot(1300, self.selectionChanged)

    @pyqtSlot()
    def on_actionAbout_triggered(self):
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Information)
        msg.setWindowTitle("About BDBag GUI")
        msg.setText("Version Information")
        msg.setInformativeText("BDBag GUI: %s\nBDBag: %s\nBagit: %s\nBagit-Profile: %s\n\n"
                               "Python: %s\nPyQt: %s\nPlatform: %s" %
                               (VERSION,
                                BDBAG_VERSION,
                                BAGIT_VERSION,
                                BAGIT_PROFILE_VERSION,
                                platform.python_version(),
                                PYQT_VERSION_STR,
                                platform.platform(aliased=True)))
        msg.setStandardButtons(QMessageBox.Ok)
        ret = msg.exec_()


# noinspection PyArgumentList,PyAttributeOutsideInit
class MainWindowUI(object):

    def setup_ui(self, MainWin):

        # Main Window
        MainWin.setObjectName("MainWindow")
        MainWin.setWindowTitle(MainWin.tr("BDBag"))
        MainWin.resize(700, 600)
        self.centralWidget = QWidget(MainWin)
        self.centralWidget.setObjectName("centralWidget")
        MainWin.setCentralWidget(self.centralWidget)
        self.verticalLayout = QVBoxLayout(self.centralWidget)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")

        # Actions

        # Materialize
        self.actionMaterialize = QAction(MainWin)
        self.actionMaterialize.setObjectName("actionMaterialize")
        self.actionMaterialize.setText(MainWin.tr("Materialize"))
        self.actionMaterialize.setToolTip(
            MainWin.tr("Extract, fetch, and validate a bag archive or directory in a single command."))
        self.actionMaterialize.setShortcut(MainWin.tr("Ctrl+M"))

        # Validate Fast
        self.actionValidateFast = QAction(MainWin)
        self.actionValidateFast.setObjectName("actionValidateFast")
        self.actionValidateFast.setText(MainWin.tr("Validate: Fast"))
        self.actionValidateFast.setToolTip(
            MainWin.tr("Perform fast validation by checking the payload file counts and sizes against the Payload-0xum "
                       "value from the bag manifest."))
        self.actionValidateFast.setShortcut(MainWin.tr("Ctrl+F"))

        # Validate Full
        self.actionValidateFull = QAction(MainWin)
        self.actionValidateFull.setObjectName("actionValidateFull")
        self.actionValidateFull.setText(MainWin.tr("Validate: Full"))
        self.actionValidateFull.setToolTip(
            MainWin.tr("Perform full validation by calculating checksums for all files and comparing them against "
                       "entries in the bag manifest(s)."))
        self.actionValidateFull.setShortcut(MainWin.tr("Ctrl+V"))

        # Fetch Missing
        self.actionFetchMissing = QAction(MainWin)
        self.actionFetchMissing.setObjectName("actionFetchMissing")
        self.actionFetchMissing.setText(MainWin.tr("Fetch: Missing"))
        self.actionFetchMissing.setToolTip(
            MainWin.tr("Fetch only those remote files that are not already present in the bag."))
        self.actionFetchMissing.setShortcut(MainWin.tr("Ctrl+P"))

        # Fetch All
        self.actionFetchAll = QAction(MainWin)
        self.actionFetchAll.setObjectName("actionFetchAll")
        self.actionFetchAll.setText(MainWin.tr("Fetch: All"))
        self.actionFetchAll.setToolTip(
            MainWin.tr("Fetch all remote files, even if they are already present in the bag."))
        self.actionFetchAll.setShortcut(MainWin.tr("Ctrl+A"))

        # Archive
        self.actionArchive = QAction(MainWin)
        self.actionArchive.setObjectName("actionArchive")
        self.actionArchive.setText(MainWin.tr("Archive"))
        self.actionArchive.setToolTip(MainWin.tr("Create a single file compressed archive of the bag."))
        self.actionArchive.setShortcut(MainWin.tr("Ctrl+Z"))

        # Create/Update
        self.actionCreateOrUpdate = QAction(MainWin)
        self.actionCreateOrUpdate.setObjectName("actionCreateOrUpdate")

        # Revert
        self.actionRevert = QAction(MainWin)
        self.actionRevert.setObjectName("actionRevert")
        self.actionRevert.setText(MainWin.tr("Revert"))
        self.actionRevert.setToolTip(
            MainWin.tr("Revert a bag directory back to a normal directory."))
        self.actionRevert.setShortcut(MainWin.tr("Ctrl+R"))

        # Delete
        self.actionDelete = QAction(MainWin)
        self.actionDelete.setObjectName("actionDelete")
        self.actionDelete.setText(MainWin.tr("Delete"))
        self.actionDelete.setToolTip(MainWin.tr("Delete"))
        self.actionDelete.setShortcut(MainWin.tr("Ctrl+D"))

        # Cancel
        self.actionCancel = QAction(MainWin)
        self.actionCancel.setObjectName("actionCancel")
        self.actionCancel.setText(MainWin.tr("Cancel"))
        self.actionCancel.setToolTip(MainWin.tr("Cancel the current background task."))
        self.actionCancel.setShortcut(MainWin.tr("Ctrl+C"))

        # Options
        self.actionOptions = QAction(MainWin)
        self.actionOptions.setObjectName("actionOptions")
        self.actionOptions.setText(MainWin.tr("Options"))
        self.actionOptions.setToolTip(MainWin.tr("Configuration options."))
        self.actionOptions.setShortcut(MainWin.tr("Ctrl+O"))

        # About
        self.actionAbout = QAction(MainWin)
        self.actionAbout.setObjectName("actionAbout")
        self.actionAbout.setText(MainWin.tr("About"))
        self.actionAbout.setToolTip(MainWin.tr("Show version information."))
        self.actionAbout.setShortcut(MainWin.tr("Ctrl+B"))

        # Tree View
        self.treeView = QTreeView(self.centralWidget)
        self.treeView.setObjectName("treeView")
        self.treeView.setStyleSheet(
            """
            QTreeView {
                    border: 2px solid grey;
                    border-radius: 5px;
            }
            """)
        self.verticalLayout.addWidget(self.treeView)

        # Log Widget

        self.logTextBrowser = log_widget.QPlainTextEditLogger(self.centralWidget)
        self.logTextBrowser.widget.setObjectName("logTextBrowser")
        self.logTextBrowser.widget.setStyleSheet(
            """
            QPlainTextEdit {
                    border: 2px solid grey;
                    border-radius: 5px;
                    background-color: lightgray;
            }
            """)
        self.verticalLayout.addWidget(self.logTextBrowser.widget)

        # Menu Bar

        self.menuBar = QMenuBar(MainWin)
        self.menuBar.setObjectName("menuBar")
        MainWin.setMenuBar(self.menuBar)

        # Bag Menu
        self.menuBag = QMenu(self.menuBar)
        self.menuBag.setObjectName("menuBag")
        self.menuBag.setTitle(MainWin.tr("Bag"))

        # Fetch Menu
        self.menuFetch = QMenu(self.menuBag)
        self.menuFetch.setObjectName("menuFetch")
        self.menuFetch.setTitle(MainWin.tr("Fetch"))
        self.menuFetch.addAction(self.actionFetchMissing)
        self.menuFetch.addAction(self.actionFetchAll)

        # Validate Menu
        self.menuValidate = QMenu(self.menuBag)
        self.menuValidate.setObjectName("menuValidate")
        self.menuValidate.setTitle(MainWin.tr("Validate"))
        self.menuValidate.addAction(self.actionValidateFast)
        self.menuValidate.addAction(self.actionValidateFull)

        # Populate Bag menu
        self.menuBar.addAction(self.menuBag.menuAction())
        self.menuBag.addAction(self.actionMaterialize)
        self.menuBag.addAction(self.menuFetch.menuAction())
        self.menuBag.addAction(self.menuValidate.menuAction())
        self.menuBag.addAction(self.actionArchive)
        self.menuBag.addAction(self.actionCreateOrUpdate)
        self.menuBag.addAction(self.actionRevert)
        self.menuBag.addAction(self.actionDelete)
        self.menuBag.addAction(self.actionCancel)
        self.menuBag.addAction(self.actionOptions)

        # Help Menu
        self.menuHelp = QMenu(self.menuBar)
        self.menuHelp.setObjectName("menuHelp")
        self.menuHelp.setTitle(MainWin.tr("Help"))
        self.menuHelp.addAction(self.actionAbout)
        self.menuBar.addAction(self.menuHelp.menuAction())

        # Tool Bar

        self.mainToolBar = QToolBar(MainWin)
        self.mainToolBar.setObjectName("mainToolBar")
        self.mainToolBar.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
        self.mainToolBar.setContextMenuPolicy(Qt.PreventContextMenu)
        MainWin.addToolBar(Qt.TopToolBarArea, self.mainToolBar)

        # Materialize
        self.mainToolBar.addAction(self.actionMaterialize)
        self.actionMaterialize.setIcon(
            self.actionMaterialize.parentWidget().style().standardIcon(getattr(QStyle, "SP_MediaPlay")))

        # Fetch
        self.mainToolBar.addAction(self.actionFetchMissing)
        self.actionFetchMissing.setIcon(
            self.actionFetchMissing.parentWidget().style().standardIcon(getattr(QStyle, "SP_ArrowDown")))
        self.mainToolBar.addAction(self.actionFetchAll)
        self.actionFetchAll.setIcon(
            self.actionFetchAll.parentWidget().style().standardIcon(getattr(QStyle, "SP_ArrowDown")))

        # Validate
        self.mainToolBar.addAction(self.actionValidateFast)
        self.actionValidateFast.setIcon(
            self.actionValidateFast.parentWidget().style().standardIcon(getattr(QStyle, "SP_DialogApplyButton")))
        self.mainToolBar.addAction(self.actionValidateFull)
        self.actionValidateFull.setIcon(
            self.actionValidateFull.parentWidget().style().standardIcon(getattr(QStyle, "SP_DialogApplyButton")))

        # Archive
        self.mainToolBar.addAction(self.actionArchive)
        self.actionArchive.setIcon(
            self.actionArchive.parentWidget().style().standardIcon(getattr(QStyle, "SP_DialogSaveButton")))

        # Create/Update
        self.mainToolBar.addAction(self.actionCreateOrUpdate)
        self.actionCreateOrUpdate.setIcon(
            self.actionCreateOrUpdate.parentWidget().style().standardIcon(getattr(QStyle, "SP_FileDialogNewFolder")))

        # Revert
        self.mainToolBar.addAction(self.actionRevert)
        self.actionRevert.setIcon(
            self.actionRevert.parentWidget().style().standardIcon(getattr(QStyle, "SP_DialogOkButton")))

        # Delete
        self.mainToolBar.addAction(self.actionDelete)
        self.actionDelete.setIcon(
            self.actionDelete.parentWidget().style().standardIcon(getattr(QStyle, "SP_DialogDiscardButton")))

        # Cancel
        self.mainToolBar.addAction(self.actionCancel)
        self.actionCancel.setIcon(
            self.actionCancel.parentWidget().style().standardIcon(getattr(QStyle, "SP_BrowserStop")))

        # Options
        self.mainToolBar.addAction(self.actionOptions)
        self.actionOptions.setIcon(
            self.actionOptions.parentWidget().style().standardIcon(getattr(QStyle, "SP_FileDialogDetailedView")))

        # Status Bar

        self.statusBar = QStatusBar(MainWin)
        self.statusBar.setToolTip("")
        self.statusBar.setStatusTip("")
        self.statusBar.setObjectName("statusBar")
        MainWin.setStatusBar(self.statusBar)

        # Progress Bar

        self.progressBar = QProgressBar(self.centralWidget)
        self.progressBar.setValue(0)
        self.progressBar.setTextVisible(False)
        self.progressBar.setObjectName("progressBar")
        self.progressBar.setStyleSheet(
            """
            QProgressBar {
                    border: 2px solid grey;
                    border-radius: 5px;
                    text-align: center;
            }
            QProgressBar::chunk {
                    background-color: cornflowerblue;
                    width: 15px;
                    margin: 0.5px;
            }
            """)
        self.verticalLayout.addWidget(self.progressBar)

        # finalize UI setup
        QMetaObject.connectSlotsByName(MainWin)

    def toggleCreateOrUpdate(self, MainWin, is_bag):
        if is_bag:
            self.actionCreateOrUpdate.setText(MainWin.tr("Update"))
            self.actionCreateOrUpdate.setToolTip(MainWin.tr("Update a bag in an existing directory."))
            self.actionCreateOrUpdate.setShortcut(MainWin.tr("Ctrl+U"))
        else:
            self.actionCreateOrUpdate.setText(MainWin.tr("Create"))
            self.actionCreateOrUpdate.setToolTip(MainWin.tr("Create a new bag from an existing directory."))
            self.actionCreateOrUpdate.setShortcut(MainWin.tr("Ctrl+N"))

    def toggleArchiveOrExtract(self, MainWin, is_bag, is_file_archive):
        self.actionArchive.setEnabled(is_bag)
        if is_bag:
            self.actionArchive.setIcon(
                self.actionArchive.parentWidget().style().standardIcon(getattr(QStyle, "SP_DialogSaveButton")))
            self.actionArchive.setText(MainWin.tr("Archive"))
            self.actionArchive.setToolTip(MainWin.tr("Create a single file compressed archive of the bag."))

        if is_file_archive:
            self.actionArchive.setEnabled(True)
            self.actionArchive.setIcon(
                self.actionArchive.parentWidget().style().standardIcon(getattr(QStyle, "SP_DialogOpenButton")))
            self.actionArchive.setText(MainWin.tr("Extract"))
            self.actionArchive.setToolTip(MainWin.tr("Extract a compressed archive file."))



#
# Copyright 2017 University of Southern California
# Distributed under the GNU GPL 3.0 license. See LICENSE for more info.
#

""" Installation script for the bdbag_gui utility.
"""
import io
import re
from setuptools import setup, find_packages

__version__ = re.search(
    r'__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
    io.open('bdbag_gui/__init__.py', encoding='utf_8_sig').read()
    ).group(1)

setup(
    name="bdbag_gui",
    description="Graphical User Interface for BDBag Utility",
    url='https://github.com/fair-research/bdbag_gui/',
    maintainer='USC Information Sciences Institute, Informatics Systems Research Division',
    maintainer_email='isrd-support@isi.edu',
    version=__version__,
    packages=find_packages(),
    package_data={'bdbag_gui': ['images/bag.ico', 'images/bag.icns', 'images/bag.png']},
    entry_points={
        'gui_scripts': [
            'bdbag-gui = bdbag_gui.__main__:main',
        ]
    },
    requires=[
        'os',
        'sys',
        'logging',
        'PyQt5'],
    install_requires=[
        'bdbag[boto,globus]>=1.6.0',
        'PyQt5'],
    license='GNU GPL 3.0',
    classifiers=[
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        "Operating System :: POSIX",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ]
)




import os
import sys
from pkg_resources import get_distribution, DistributionNotFound

__version__ = "1.1.0"

try:
    version = get_distribution("bdbag_gui").version
    VERSION = version + '' if not getattr(sys, 'frozen', False) else version + '-frozen'
except DistributionNotFound:
    VERSION = __version__ + '-dev' if not getattr(sys, 'frozen', False) else __version__ + '-frozen'



import os
import logging
from PyQt5.QtCore import Qt, pyqtSlot
from PyQt5.QtWidgets import QDialog, QVBoxLayout, QHBoxLayout, QLabel, QLineEdit, QPushButton, QFileDialog, \
    QGroupBox, QCheckBox, QRadioButton, QMessageBox, QDialogButtonBox, qApp
from .json_editor import JSONEditor
from bdbag.bdbag_config import write_config, DEFAULT_CONFIG_PATH, DEFAULT_CONFIG_FILE, DEFAULT_KEYCHAIN_FILE

DEFAULT_OPTIONS_FILE = os.path.join(DEFAULT_CONFIG_PATH, 'bdbag_gui.json')
DEFAULT_OPTIONS = {
    "archive_format": "zip",
    "archive_extract_dir": "",
    "bag_config_file_path": DEFAULT_CONFIG_FILE,
    "bag_keychain_file_path": DEFAULT_KEYCHAIN_FILE
}


def warningMessageBox(parent, text, detail):
    msg = QMessageBox(parent)
    msg.setIcon(QMessageBox.Warning)
    msg.setWindowTitle("Attention Required")
    msg.setText(text)
    msg.setInformativeText(detail)
    msg.exec_()


class OptionsDialog(QDialog):
    def __init__(self, parent):
        super(OptionsDialog, self).__init__(parent)
        self.config_file = parent.options.get("bag_config_file_path") or DEFAULT_OPTIONS["bag_config_file_path"]
        self.keychain_file = parent.options.get("bag_keychain_file_path") or DEFAULT_OPTIONS["bag_keychain_file_path"]
        self.archive_extract_dir = parent.options.get("archive_extract_dir") or ""
        self.archive_format = parent.options.get("archive_format") or DEFAULT_OPTIONS["archive_format"]
        self.setWindowTitle("Options")
        self.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint)
        self.setMinimumWidth(600)
        layout = QVBoxLayout(self)
        layout.addStretch(1)

        # Configuration Group
        self.configGroupLayout = QVBoxLayout()
        self.configGroupBox = QGroupBox("Bag configuration:", self)
        self.configGroupBox.setLayout(self.configGroupLayout)
        layout.addWidget(self.configGroupBox)

        # BDBag config file
        self.configFileLayout = QHBoxLayout()
        self.configFilePathLabel = QLabel("Configuration File:")
        self.configFileLayout.addWidget(self.configFilePathLabel)
        self.configFilePathTextBox = QLineEdit()
        self.configFilePathTextBox.setReadOnly(True)
        self.configFilePathTextBox.setText(os.path.normpath(self.config_file))
        self.configFileLayout.addWidget(self.configFilePathTextBox)
        self.configFileBrowseButton = QPushButton("Change", parent)
        self.configFileBrowseButton.clicked.connect(self.onConfigChange)
        self.configFileLayout.addWidget(self.configFileBrowseButton)
        self.configFileEditButton = QPushButton("Edit", parent)
        self.configFileEditButton.clicked.connect(self.onConfigEdit)
        self.configFileLayout.addWidget(self.configFileEditButton)
        self.configGroupLayout.addLayout(self.configFileLayout)

        # BDBag keychain file
        self.keychainFileLayout = QHBoxLayout()
        self.keychainFilePathLabel = QLabel("Keychain File:")
        self.keychainFileLayout.addWidget(self.keychainFilePathLabel)
        self.keychainFilePathTextBox = QLineEdit()
        self.keychainFilePathTextBox.setReadOnly(True)
        self.keychainFilePathTextBox.setText(os.path.normpath(self.keychain_file))
        self.keychainFileLayout.addWidget(self.keychainFilePathTextBox)
        self.keychainFileBrowseButton = QPushButton("Change", parent)
        self.keychainFileBrowseButton.clicked.connect(self.onKeychainChange)
        self.keychainFileLayout.addWidget(self.keychainFileBrowseButton)
        self.keychainFileEditButton = QPushButton("Edit", parent)
        self.keychainFileEditButton.clicked.connect(self.onKeychainEdit)
        self.keychainFileLayout.addWidget(self.keychainFileEditButton)
        self.configGroupLayout.addLayout(self.keychainFileLayout)

        # Archive/Extract Group
        self.archiveGroupLayout = QVBoxLayout()
        self.archiveGroupBox = QGroupBox("Bag archiving and extraction:", self)
        self.archiveGroupBox.setLayout(self.archiveGroupLayout)
        layout.addWidget(self.archiveGroupBox)

        # Extraction directory path
        self.extractPathLayout = QHBoxLayout()
        self.extractPathLabel = QLabel("Archive extraction directory:")
        self.extractPathLayout.addWidget(self.extractPathLabel)
        self.extractPathTextBox = QLineEdit()
        self.extractPathTextBox.setReadOnly(True)
        self.extractPathTextBox.setText(os.path.normpath(self.archive_extract_dir))
        self.extractPathLayout.addWidget(self.extractPathTextBox)
        self.extractPathBrowseButton = QPushButton("Change", parent)
        self.extractPathBrowseButton.clicked.connect(self.onExtractPathChange)
        self.extractPathLayout.addWidget(self.extractPathBrowseButton)
        self.archiveGroupLayout.addLayout(self.extractPathLayout)

        # Archive format radio group
        self.archiveFormatLayout = QHBoxLayout()
        self.archiveFormatLabel = QLabel("Archive creation format:")
        self.archiveFormatLayout.addWidget(self.archiveFormatLabel)
        self.archiveFormatLayout.insertSpacing(1, 20)
        self.archiveFormatZipButton = QRadioButton("&ZIP")
        self.archiveFormatZipButton.setChecked(self.archive_format.lower() == "zip")
        self.archiveFormatZipButton.toggled.connect(self.onArchiveFormatChanged)
        self.archiveFormatLayout.addWidget(self.archiveFormatZipButton)
        self.archiveFormatTGZButton = QRadioButton("T&GZ")
        self.archiveFormatTGZButton.setChecked(self.archive_format.lower() == "tgz")
        self.archiveFormatTGZButton.toggled.connect(self.onArchiveFormatChanged)
        self.archiveFormatLayout.addWidget(self.archiveFormatTGZButton)
        self.archiveFormatBZ2Button = QRadioButton("&BZ2")
        self.archiveFormatBZ2Button.setChecked(self.archive_format.lower() == "bz2")
        self.archiveFormatBZ2Button.toggled.connect(self.onArchiveFormatChanged)
        self.archiveFormatLayout.addWidget(self.archiveFormatBZ2Button)
        self.archiveFormatTARButton = QRadioButton("&TAR")
        self.archiveFormatTARButton.setChecked(self.archive_format.lower() == "tar")
        self.archiveFormatTARButton.toggled.connect(self.onArchiveFormatChanged)
        self.archiveFormatLayout.addWidget(self.archiveFormatTARButton)
        self.archiveGroupLayout.addLayout(self.archiveFormatLayout)

        # Miscellaneous Group
        self.miscGroupBox = QGroupBox("Miscellaneous:", self)
        self.miscLayout = QHBoxLayout()
        self.debugCheckBox = QCheckBox("Debug logging")
        self.debugCheckBox.setChecked(True if logging.getLogger().getEffectiveLevel() == logging.DEBUG else False)
        self.miscLayout.addWidget(self.debugCheckBox)
        self.miscGroupBox.setLayout(self.miscLayout)
        layout.addWidget(self.miscGroupBox)

        # Button Box
        self.buttonBox = QDialogButtonBox(parent)
        self.buttonBox.setObjectName("buttonBox")
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel | QDialogButtonBox.Ok)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)
        layout.addWidget(self.buttonBox)

    @pyqtSlot()
    def onConfigChange(self):
        current_path = os.path.normpath(self.configFilePathTextBox.text())
        new_path = QFileDialog.getOpenFileName(self,
                                               "Select Configuration File",
                                               current_path,
                                               "Configuration Files (*.json)")
        if new_path[0]:
            new_path = os.path.normpath(new_path[0])
            if new_path != current_path:
                self.configFilePathTextBox.setText(new_path)
                self.config_file = new_path

    @pyqtSlot()
    def onConfigEdit(self):
        configEditor = JSONEditor(self, self.configFilePathTextBox.text(), "Configuration Editor")
        configEditor.exec_()
        if configEditor.isModified():
            path = self.configFilePathTextBox.text()
            self.config_file = path
        del configEditor

    @pyqtSlot()
    def onKeychainChange(self):
        current_path = os.path.normpath(self.keychainFilePathTextBox.text())
        new_path = QFileDialog.getOpenFileName(self, "Select Keychain File", current_path, "Keychain Files (*.json)")
        if new_path[0]:
            new_path = os.path.normpath(new_path[0])
            if new_path != current_path:
                self.keychainFilePathTextBox.setText(new_path)
                self.keychain_file = new_path

    @pyqtSlot()
    def onKeychainEdit(self):
        configEditor = JSONEditor(self, self.keychainFilePathTextBox.text(), "Keychain Editor")
        configEditor.exec_()
        if configEditor.isModified():
            path = self.keychainFilePathTextBox.text()
            self.keychain_file = path
        del configEditor

    @pyqtSlot()
    def onExtractPathChange(self):
        current_path = os.path.normpath(self.configFilePathTextBox.text())
        dialog = QFileDialog()
        path = dialog.getExistingDirectory(self, "Select Directory", current_path, QFileDialog.ShowDirsOnly)
        if not path:
            return
        new_path = os.path.normpath(path)
        if new_path != current_path:
            self.extractPathTextBox.setText(new_path)
            self.archive_extract_dir = new_path

    @pyqtSlot(bool)
    def onArchiveFormatChanged(self, checked):
        if checked:
            if self.archiveFormatZipButton.isChecked():
                self.archive_format = "zip"
            elif self.archiveFormatTGZButton.isChecked():
                self.archive_format = "tgz"
            elif self.archiveFormatBZ2Button.isChecked():
                self.archive_format = "bz2"
            elif self.archiveFormatTARButton.isChecked():
                self.archive_format = "tar"

    @pyqtSlot()
    def restoreDefaults(self):
        self.config_file = DEFAULT_OPTIONS["bag_config_file_path"]
        self.configFilePathTextBox.setText(self.config_file)
        self.keychain_file = DEFAULT_OPTIONS["bag_keychain_file_path"]
        self.keychainFilePathTextBox.setText(self.keychain_file)
        self.archive_extract_dir = DEFAULT_OPTIONS["archive_extract_dir"]
        self.archive_format = DEFAULT_OPTIONS["archive_format"]

    @staticmethod
    def getOptions(parent):
        dialog = OptionsDialog(parent)
        ret = dialog.exec_()
        if QDialog.Accepted == ret:
            debug = dialog.debugCheckBox.isChecked()
            logging.getLogger().setLevel(logging.DEBUG if debug else logging.INFO)
            dirty = False
            if dialog.config_file != parent.options["bag_config_file_path"]:
                parent.options["bag_config_file_path"] = dialog.config_file
                dirty = True
            if dialog.keychain_file != parent.options["bag_keychain_file_path"]:
                parent.options["bag_keychain_file_path"] = dialog.keychain_file
                dirty = True
            if dialog.archive_extract_dir != parent.options["archive_extract_dir"]:
                parent.options["archive_extract_dir"] = dialog.archive_extract_dir
                dirty = True
            if dialog.archive_format != parent.options["archive_format"]:
                parent.options["archive_format"] = dialog.archive_format
                dirty = True
            if dirty:
                parent.saveOptions()
        del dialog

